#include "xsysmonpsv.h"

XSysMonPsv_Config XSysMonPsv_ConfigTable[] __attribute__ ((section (".drvcfg_sec"))) = {

	{
		"xlnx,versal-sysmon", /* compatible */
		0xf1270000, /* reg */
		0xffff, /* interrupts */
		0xffff, /* interrupt-parent */
		{
				0, /* reg */
				5, /* reg */
				1, /* reg */
				2, /* reg */
				3, /* reg */
				4, /* reg */
				6, /* reg */
		}, /* child,required */
	},
	 {
		 NULL
	}
};

const char *XSysMonPsv_Supply_Arr[] = {
	"VCCAUX",
	"VCC_SOC",
	"VCCAUX_PMC",
	"VCC_PMC",
	"VCC_PSFP",
	"VCC_PSLP",
	"VP_VN",
};
